<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.2                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: geoip.inc.php 2182 2005-03-17 11:46:04Z andrew $
*/

/**
 * @package    MaxDelivery
 * @subpackage GeoTargeting
 * @author     Andrew Hill <andrew@awarez.net>
 *
 * @TODO Not all MaxMind LLC databases have been tested yet!
 *
 * File to get GeoTargeting information directly from the MaxMind LLC
 * database file, without having it accessed via the C/mod_apache
 * interface.
 */

$phpAds_geoPluginID = 'geoip';

/**
 * A method to return information about the plugin.
 *
 * @return string A string describing the plugin.
 */
function phpAds_geoip_getInfo()
{
	return (array (
		'name'	    => 'MaxMind GeoIP Country Edition',
		'db'	    => true,
		'country'   => true,
		'continent' => true,
		'region'    => false
	));
}

/**
 * The function to look up the GeoTargeting information, based on
 * the IP address.
 *
 * @param string $ip The dotted quad formatted IP address.
 * @@return array An array that will contain the results of the GeoTargeting lookup.
 */
function phpAds_geoip_getGeo($ip)
{
    $config =& $GLOBALS['phpAds_config'];
    
    // Use the MaxMind GeoIP Country Database
    if ($config['geotracking_geoip_country_location'] == '') {
        // Use the free version of the database distributed with Max
        $config['geotracking_geoip_country_location'] =
            phpAds_path . '/libraries/geotargeting/maxmind/FreeGeoIPCountry.dat';
    }
    if ($config['geotracking_geoip_country_location'] != '') {
        require_once phpAds_path . '/libraries/geotargeting/maxmind/geoip.inc';
        $gi = geoip_open($config['geotracking_geoip_country_location'], GEOIP_STANDARD);
        $result['country_code'] = geoip_country_code_by_addr($gi, $ip);
        $result['country_name'] = geoip_country_name_by_addr($gi, $ip);
        geoip_close($gi);
    }
    
    // Use the MaxMind GeoIP Region Database
    if ($config['geotracking_geoip_region_location'] != '') {
        require_once phpAds_path . '/libraries/geotargeting/maxmind/geoip.inc';
        $gi = geoip_open($config['geotracking_geoip_region_location'], GEOIP_STANDARD);
        list($country, $region) = geoip_region_by_addr($gi, $ip);
        if (is_null($result['country_code']) || ($result['country_code'] == '')) {
            $result['country_code'] = $country;
        }
        $result['region'] = $region;
        geoip_close($gi);
    }
    
    // Use the MaxMind GeoIP City Database
    if ($config['geotracking_geoip_city_location'] != '') {
        require_once phpAds_path . '/libraries/geotargeting/maxmind/geoipcity.inc';
        $gi = geoip_open($config['geotracking_geoip_city_location'], GEOIP_STANDARD);
        $record = geoip_record_by_addr($gi, $ip);
        if (is_null($result['country_code']) || ($result['country_code'] == '')) {
            $result['country_code'] = $record->country_code;
        }
        if (is_null($result['country_name']) || ($result['country_name'] == '')) {
            $result['country_name'] = $record->country_name;
        }
        if (is_null($result['region']) || ($result['region'] == '')) {
            $result['region'] = $record->region;
        }
        $result['city'] = $record->city;
        $result['postal_code'] = $record->postal_code;
        $result['latitude'] = (string) $record->latitude;
        $result['longitude'] = (string) $record->longitude;
        $result['dma_code'] = $record->dma_code;
        $result['area_code'] = $record->area_code;
        geoip_close($gi);
    }
    
    // Use the MaxMind GeoIP Area Code Database
    if ($config['geotracking_geoip_area_location'] != '') {
        require_once phpAds_path . '/libraries/geotargeting/maxmind/geoipcity.inc';
        $gi = geoip_open($config['geotracking_geoip_area_location'], GEOIP_STANDARD);
        $record = geoip_record_by_addr($gi, $ip);
        if (is_null($result['country_code']) || ($result['country_code'] == '')) {
            $result['country_code'] = $record->country_code;
        }
        if (is_null($result['country_name']) || ($result['country_name'] == '')) {
            $result['country_name'] = $record->country_name;
        }
        if (is_null($result['region']) || ($result['region'] == '')) {
            $result['region'] = $record->region;
        }
        if (is_null($result['area_code']) || ($result['area_code'] == '')) {
            $result['area_code'] = $record->area_code;
        }
        geoip_close($gi);
        
    }
    
    // Use the MaxMind GeoIP DMA Code Database
    if ($config['geotracking_geoip_dma_location'] != '') {
        require_once phpAds_path . '/libraries/geotargeting/maxmind/geoipcity.inc';
        $gi = geoip_open($config['geotracking_geoip_dma_location'], GEOIP_STANDARD);
        $record = geoip_record_by_addr($gi, $ip);
        if (is_null($result['country_code']) || ($result['country_code'] == '')) {
            $result['country_code'] = $record->country_code;
        }
        if (is_null($result['country_name']) || ($result['country_name'] == '')) {
            $result['country_name'] = $record->country_name;
        }
        if (is_null($result['region']) || ($result['region'] == '')) {
            $result['region'] = $record->region;
        }
        if (is_null($result['dma_code']) || ($result['dma_code'] == '')) {
            $result['dma_code'] = $record->dma_code;
        }
        geoip_close($gi);
        
    }
    
    // Use the MaxMind GeoIP Organization
    if ($config['geotracking_geoip_org_location'] != '') {
        require_once phpAds_path . '/libraries/geotargeting/maxmind/geoip.inc';
        $gi = geoip_open($config['geotracking_geoip_org_location'], GEOIP_STANDARD);
        $result['organisation'] = geoip_org_by_addr($gi, $ip);
        geoip_close($gi);
    }
    
    // Use the MaxMind GeoIP ISP
    if ($config['geotracking_geoip_isp_location'] != '') {
        require_once phpAds_path . '/libraries/geotargeting/maxmind/geoip.inc';
        $gi = geoip_open($config['geotracking_geoip_isp_location'], GEOIP_STANDARD);
        $result['isp'] = geoip_org_by_addr($gi, $ip);
        geoip_close($gi);
    }
    
    // Use the MaxMind GeoIP Netspeed
    if ($config['geotracking_geoip_netspeed_location'] != '') {
        require_once phpAds_path . '/libraries/geotargeting/maxmind/geoip.inc';
        $gi = geoip_open($config['geotracking_geoip_netspeed_location'], GEOIP_STANDARD);
        $netspeed = geoip_country_id_by_addr($gi, $ip);
        if ($netspeed == GEOIP_UNKNOWN_SPEED) {
            $result['netspeed'] = "Unknown";
        } else if ($netspeed == GEOIP_DIALUP_SPEED) {
            $result['netspeed'] = "Dailup";
        } else if ($netspeed == GEOIP_CABLEDSL_SPEED) {
            $result['netspeed'] = "Cable/DSL";
        } else if ($netspeed == GEOIP_CORPORATE_SPEED) {
            $result['netspeed'] = "Corporate";
        }
        geoip_close($gi);
    }
    return $result;
}

?>
