<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.2                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: StatisticsAdServer.php 2274 2005-03-29 17:27:57Z chris $
*/

/**
 * @package    MaxDal
 * @subpackage Statistics
 * @author     Andrew Hill <andrew@awarez.net>
 */

require_once maintenancePath . '/maintenance-common.php';
require_once maintenancePath . '/libraries/max/Maintenance.php';
require_once maintenancePath . '/libraries/max/Dal/StatisticsCommon.php';
require_once maintenancePath . '/libraries/pear/Date.php';
 
/**
 * The data access layer code for summarising raw data into statistics, for 
 * the AdServer module.
 */
class Dal_Statistics_AdServer extends Dal_Statistics_Common
{

    /**
     * The constructor method.
     *
     * @uses Dal_Statistics_Common::Dal_Statistics_Common()
     */
    function Dal_Statistics_AdServer()
    {
        parent::Dal_Statistics_Common();
    }
    
    /**
     * The singleton method to create or return a single instance of the class.
     *
     * @static
     * @return Dal_Statistics_AdServer The returned Dal_Statistics_AdServer object.
     */
    function &singleton() {
        static $instance;
        if (!isset($instance)) {
            $instance = new Dal_Statistics_AdServer();
        }
        return $instance;
    }
    
    /**
     * A method to find the last time that maintenance statistics was run.
     *
     * @param integer $type The update type that occurred - that is, 0 if
     *                      the update was done on the basis of the 
     *                      operation interval, or 1 if the update was
     *                      done on the basis of the hour.
     * @param Date $now An optional Date, used to specify the "current time", and
     *                  to limit the method to only look for past maintenance
     *                  statistics runs before this date. Normally only used
     *                  to assist with re-generation of statistics in the event
     *                  of faulty raw tables.
     * @return Date A Date representing the date up to which the statistics
     *              have been summarised, for the specified update type, or
     *              the appropriate date based on raw data if maintenance
     *              statistics has never been run for the Max module before.
     *              Returns null if no raw data is available.
     */
    function getMaintenanceStatisticsLastRunInfo($type, $now = null)
    {
        $table = $this->conf['table']['data_raw_ad_impression'];
        return $this->_getMaintenanceStatisticsLastRunInfo($type, $table, $now);
    }
    
    /**
     * A private function to do the job of
     * {@link Dal_Statistics_AdServer::getMaintenanceStatisticsLastRunInfo()}, but with an
     * extra parameter to specify the raw table to look in, in the case of
     * maintenance statistics not having been run before.
     *
     * @private
     * @param integer $type The update type that occurred - that is, 0 if
     *                      the update was done on the basis of the 
     *                      operation interval, or 1 if the update was
     *                      done on the basis of the hour.
     * @param string $rawTable The raw table to use in case of no previous run.
     * @param Date $now An optional Date, used to specify the "current time", and
     *                  to limit the method to only look for past maintenance
     *                  statistics runs before this date. Normally only used
     *                  to assist with re-generation of statistics in the event
     *                  of faulty raw tables.
     * @return Date A Date representing the date up to which the statistics
     *              have been summarised, for the specified update type, or
     *              the appropriate date based on raw data if maintenance
     *              statistics has never been run for the Max module before.
     *              Returns null if no raw data is available.
     */
    function _getMaintenanceStatisticsLastRunInfo($type, $rawTable, $now = null) {
        if ($type == 0) {
            $whereClause = 'WHERE (lms.adserver_run_type = 0 OR lms.adserver_run_type = 2)';
        } else if ($type == 1) {
            $whereClause = 'WHERE (lms.adserver_run_type = 1 OR lms.adserver_run_type = 2)';
        } else {
            debug('Invalid update type value ' . $type . ': Must be 0 or 1', PEAR_LOG_ERROR);
            debug('Aborting script execution', PEAR_LOG_ERROR);
            exit();
        }
        if (!is_null($now)) {
            // Limit to past maintenance statistics runs before this Date
            $whereClause .= ' AND updated_to < ' . "'" . $now->format('%Y-%m-%d %H:%M:%S') . "'";
        }
        $query = "
            SELECT
                lms.updated_to AS date
            FROM 
                {$this->conf['table']['log_maintenance_statistics']} AS lms
            $whereClause
            ORDER BY date DESC
            LIMIT 1";
        debug('Getting the details of when maintenance statistics last ran for the AdServer module',
              PEAR_LOG_DEBUG);
        $this->execute($query, $result);
        $row = $this->fetchResultAsArray($result);
        if ($row) {
            $date = new Date($row['date']);
            return $date;
        } else {
            // Maintenance statistics has never been run - query the raw data
            $query = "
                SELECT
                    raw.date_time AS date
                FROM
                    $rawTable AS raw
                ORDER BY date ASC
                LIMIT 1";
            debug('Getting the details of when the first ad impression is for the AdServer module',
                  PEAR_LOG_DEBUG);
            $errors = array(1146);   // Permit the table does not exist error
            $errorID = $this->executeIgnoreErrors($query, $result, $errors);
            if ($errorID == 0) {
                $row = $this->fetchResultAsArray($result);
                if ($row) {
                    // Convert the impression time into the end of the previous operation interval or hour
                    $date = new Date($row['date']);
                    if ($type == 0) {
                        list($date, $_) = Maintenance::convertDateToOperationIntervalStartAndEndDates($date);
                        $date->subtractSeconds(1);
                        return $date;
                    } else {
                        $date = new Date($date->format('%Y-%m-%d %H:00:00'));
                        $date->subtractSeconds(1);
                        return $date;
                    }
                }
            }
            // No raw data was found
            return null;
        }
    }
    
    /**
     * A method for summarising requests into a temporary table.
     *
     * @param Date $start The start date/time to summarise from.
     * @param Date $end The end date/time to summarise to.
     * @return integer The number of request rows summarised.
     */
    function summariseRequests($start, $end)
    {
        // Check the start and end dates
        if (!Maintenance::checkIntervalDates($start, $end, $this->conf['maintenance']['operation_interval'])) {
            return 0;
        }
        // Get the start and end dates of the operation interval ID
        list($operationIntervalStart, $operationIntervalEnd) =
            Maintenance::convertDateToOperationIntervalStartAndEndDates($start);
        // Get the operation interval ID
        $operationIntervalID = Maintenance::convertDateToOperationIntervalID($operationIntervalStart);
        // Create the temporary summary table
        $this->tempTables->createTable('tmp_ad_request');
        // Summarise the requests
        $requestTable = $this->conf['table']['data_raw_ad_request'];
        $query = "
            INSERT INTO
                tmp_ad_request
                (
                    day,
                    hour,
                    operation_interval,
                    operation_interval_id,
                    interval_start,
                    interval_end,
                    ad_id,
                    creative_id,
                    zone_id,
                    requests
                )
            SELECT
                DATE_FORMAT(drar.date_time, '%Y-%m-%d') AS day,
                DATE_FORMAT(drar.date_time, '%k') AS hour,
                {$this->conf['maintenance']['operation_interval']} AS operation_interval,
                $operationIntervalID AS operation_interval_id,
                '".$operationIntervalStart->format('%Y-%m-%d %H:%M:%S')."' AS interval_start,
                '".$operationIntervalEnd->format('%Y-%m-%d %H:%M:%S')."' AS interval_end,
                drar.ad_id AS ad_id,
                drar.creative_id AS creative_id,
                drar.zone_id AS zone_id,
                COUNT(*) AS requests
            FROM
                $requestTable AS drar
            WHERE
                drar.date_time >= ".$start->format('%Y%m%d%H%M%S')."
                AND drar.date_time <= ".$end->format('%Y%m%d%H%M%S')."
            GROUP BY
                day, hour, ad_id, creative_id, zone_id";
        debug("Summarising ad requests from the $requestTable table", PEAR_LOG_DEBUG);
        $this->execute($query, $result);
        return $this->affectedRows();
    }
    
    /**
     * A method for summarising impressions into a temporary table.
     *
     * @param Date $start The start date/time to summarise from.
     * @param Date $end The end date/time to summarise to.
     * @return integer The number of impression rows summarised.
     */
    function summariseImpressions($start, $end)
    {
        // Check the start and end dates
        if (!Maintenance::checkIntervalDates($start, $end, $this->conf['maintenance']['operation_interval'])) {
            return 0;
        }
        // Get the start and end dates of the operation interval ID
        list($operationIntervalStart, $operationIntervalEnd) =
            Maintenance::convertDateToOperationIntervalStartAndEndDates($start);
        // Get the operation interval ID
        $operationIntervalID = Maintenance::convertDateToOperationIntervalID($operationIntervalStart);
        // Create the temporary summary table
        $this->tempTables->createTable('tmp_ad_impression');
        // Summarise the impressions
        $impressionTable = $this->conf['table']['data_raw_ad_impression'];
        $query = "
            INSERT INTO
                tmp_ad_impression
                (
                    day,
                    hour,
                    operation_interval,
                    operation_interval_id,
                    interval_start,
                    interval_end,
                    ad_id,
                    creative_id,
                    zone_id,
                    impressions
                )
            SELECT
                DATE_FORMAT(drai.date_time, '%Y-%m-%d') AS day,
                DATE_FORMAT(drai.date_time, '%k') AS hour,
                {$this->conf['maintenance']['operation_interval']} AS operation_interval,
                $operationIntervalID AS operation_interval_id,
                '".$operationIntervalStart->format('%Y-%m-%d %H:%M:%S')."' AS interval_start,
                '".$operationIntervalEnd->format('%Y-%m-%d %H:%M:%S')."' AS interval_end,
                drai.ad_id AS ad_id,
                drai.creative_id AS creative_id,
                drai.zone_id AS zone_id,
                COUNT(*) AS impressions
            FROM
                $impressionTable AS drai
            WHERE
                drai.date_time >= ".$start->format('%Y%m%d%H%M%S')."
                AND drai.date_time <= ".$end->format('%Y%m%d%H%M%S')."
            GROUP BY
                day, hour, ad_id, creative_id, zone_id";
        debug("Summarising ad impressions from the $impressionTable table", PEAR_LOG_DEBUG);
        $this->execute($query, $result);
        return $this->affectedRows();
    }

    /**
     * A method for summarising clicks into a temporary table.
     *
     * @param Date $start The start date/time to summarise from.
     * @param Date $end The end date/time to summarise to.
     * @return integer The number of click rows summarised.
     */    
    function summariseClicks($start, $end)
    {
        // Check the start and end dates
        if (!Maintenance::checkIntervalDates($start, $end, $this->conf['maintenance']['operation_interval'])) {
            return 0;
        }
        // Get the start and end dates of the operation interval ID
        list($operationIntervalStart, $operationIntervalEnd) =
            Maintenance::convertDateToOperationIntervalStartAndEndDates($start);
        // Get the operation interval ID
        $operationIntervalID = Maintenance::convertDateToOperationIntervalID($operationIntervalStart);
        // Create the temporary summary table
        $this->tempTables->createTable('tmp_ad_click');
        // Summarise the clicks
        $clickTable = $this->conf['table']['data_raw_ad_click'];
        $query = "
            INSERT INTO
                tmp_ad_click
                (
                    day,
                    hour,
                    operation_interval,
                    operation_interval_id,
                    interval_start,
                    interval_end,
                    ad_id,
                    creative_id,
                    zone_id,
                    clicks
                )
            SELECT
                DATE_FORMAT(drac.date_time, '%Y-%m-%d') AS day,
                DATE_FORMAT(drac.date_time, '%k') AS hour,
                {$this->conf['maintenance']['operation_interval']} AS operation_interval,
                $operationIntervalID AS operation_interval_id,
                '".$operationIntervalStart->format('%Y-%m-%d %H:%M:%S')."' AS interval_start,
                '".$operationIntervalEnd->format('%Y-%m-%d %H:%M:%S')."' AS interval_end,
                drac.ad_id AS ad_id,
                drac.creative_id AS creative_id,
                drac.zone_id AS zone_id,
                COUNT(*) AS clicks
            FROM
                $clickTable AS drac
            WHERE
                drac.date_time >= ".$start->format('%Y%m%d%H%M%S')."
                AND drac.date_time <= ".$end->format('%Y%m%d%H%M%S')."
            GROUP BY
                day, hour, ad_id, creative_id, zone_id";
        debug("Summarising ad clicks from the $clickTable table", PEAR_LOG_DEBUG);
        $this->execute($query, $result);
        return $this->affectedRows();
    }
    
    /**
     * A method for summarising connections into a temporary table.
     *
     * @param Date $start The start date/time to summarise from.
     * @param Date $end The end date/time to summarise to.
     * @return integer The number of connection rows summarised.
     */
    function summariseConnections($start, $end)
    {
        if (!$this->conf['modules']['Tracker']) {
            return 0;
        }
        // Check the start and end dates
        if (!Maintenance::checkIntervalDates($start, $end, $this->conf['maintenance']['operation_interval'])) {
            return 0;
        }
        // Get the start and end dates of the operation interval ID
        list($operationIntervalStart, $operationIntervalEnd) =
            Maintenance::convertDateToOperationIntervalStartAndEndDates($start);
        // Get the operation interval ID
        $operationIntervalID = Maintenance::convertDateToOperationIntervalID($operationIntervalStart);
        // Create the temporary summary tables
        $this->tempTables->createTable('tmp_tracker_impression_ad_impression_connection');
        $this->tempTables->createTable('tmp_tracker_impression_ad_click_connection');
        $this->tempTables->createTable('tmp_ad_connection');       
        // Insert tracker impressions that may connect to ad impressions into
        // the temporary tmp_tracker_impression_ad_impression_connection table
        $query = "
            INSERT INTO
                tmp_tracker_impression_ad_impression_connection
                (
                    server_raw_tracker_impression_id,
                    server_raw_ip,
                    tracker_id,
                    campaign_id,
                    operation_interval,
                    operation_interval_id,
                    interval_start,
                    interval_end,
                    viewer_id,
                    date_time,
                    connection_type,
                    connection_window,
                    conversion
                )
            SELECT
                drti.server_raw_tracker_impression_id AS server_raw_tracker_impression_id,
                drti.server_raw_ip AS server_raw_ip,
                drti.tracker_id AS tracker_id,
                ct.campaignid AS campaign_id,
                {$this->conf['maintenance']['operation_interval']} AS operation_interval,
                $operationIntervalID AS operation_interval_id,
                '".$operationIntervalStart->format('%Y-%m-%d %H:%M:%S')."' AS interval_start,
                '".$operationIntervalEnd->format('%Y-%m-%d %H:%M:%S')."' AS interval_end,
                drti.viewer_id AS viewer_id,
                drti.date_time AS date_time,
                '0' AS connection_type,
                ct.viewwindow AS connection_window,
                ct.logstats AS conversion
            FROM
                {$this->conf['table']['data_raw_tracker_impression']} AS drti,        
                {$this->conf['table']['campaigns_trackers']} AS ct
            WHERE        
                drti.tracker_id = ct.trackerid
                AND drti.viewer_id != ''
                AND drti.date_time >= ".$start->format('%Y%m%d%H%M%S')."
                AND drti.date_time <= ".$end->format('%Y%m%d%H%M%S')."
                AND ct.viewwindow > 0";
        debug('Selecting tracker impressions that may connect to ad impression into the ' .
              'tmp_tracker_impression_ad_impression_connection temporary table', PEAR_LOG_DEBUG);
        $this->execute($query, $result);
        // Connect the tracker impressions with ad impressions, where possible
        $query = "
            INSERT INTO
                tmp_ad_connection
                (
                    server_raw_tracker_impression_id,
                    server_raw_ip,
                    date_time,
                    operation_interval,
                    operation_interval_id,
                    interval_start,
                    interval_end,
                    connection_viewer_id,
                    connection_viewer_session_id,
                    connection_date_time,
                    connection_ad_id,
                    connection_creative_id,
                    connection_zone_id,
                    connection_channel,
                    connection_language,
                    connection_ip_address,
                    connection_host_name,
                    connection_country,
                    connection_https,
                    connection_domain,
                    connection_page,
                    connection_query,
                    connection_referer,
                    connection_search_term,
                    connection_user_agent,
                    connection_os,
                    connection_browser,
                    connection_type,
                    connection_window,
                    conversion
                )
            SELECT
                ttiaic.server_raw_tracker_impression_id AS server_raw_tracker_impression_id,
                ttiaic.server_raw_ip AS server_raw_ip,
                ttiaic.date_time AS date_time,
                ttiaic.operation_interval AS operation_interval,
                ttiaic.operation_interval_id AS operation_interval_id,
                ttiaic.interval_start AS interval_start,
                ttiaic.interval_end AS interval_end,
                drai.viewer_id AS connection_viewer_id,
                drai.viewer_session_id AS connection_viewer_session_id,
                drai.date_time AS connection_date_time,
                drai.ad_id AS connection_ad_id,
                drai.creative_id AS connection_creative_id,
                drai.zone_id AS connection_zone_id,
                drai.channel AS connection_channel,
                drai.language AS connection_language,
                drai.ip_address AS connection_ip_address,
                drai.host_name AS connection_host_name,
                drai.country AS connection_country,
                drai.https AS connection_https,
                drai.domain AS connection_domain,
                drai.page AS connection_page,
                drai.query AS connection_query,
                drai.referer AS connection_referer,
                drai.search_term AS connection_search_term,
                drai.user_agent AS connection_user_agent,
                drai.os AS connection_os,
                drai.browser AS connection_browser,
                ttiaic.connection_type AS connection_type,
                ttiaic.connection_window AS connection_window,
                ttiaic.conversion AS conversion
            FROM
                tmp_tracker_impression_ad_impression_connection AS ttiaic,
                {$this->conf['table']['data_raw_ad_impression']} AS drai,
                {$this->conf['table']['banners']} AS b,
                {$this->conf['table']['campaigns_trackers']} AS ct
            WHERE
                ttiaic.viewer_id = drai.viewer_id
                AND drai.ad_id = b.bannerid
                AND b.campaignid = ct.campaignid
                AND ct.trackerid = ttiaic.tracker_id
                AND ttiaic.campaign_id = ct.campaignid
                AND ttiaic.date_time < DATE_ADD(drai.date_time, INTERVAL ttiaic.connection_window SECOND)
                AND ttiaic.date_time >= drai.date_time";
        debug('Connecting tracker impressions with ad impressions, where possible', PEAR_LOG_DEBUG);
        $this->execute($query, $result);
        $rows += $this->affectedRows();
        // Drop the temporary table for storing tracker impressions that may connect to ad impressions
        $this->tempTables->dropTempTable('tmp_tracker_impression_ad_impression_connection');
        // Insert tracker impressions that may connect to ad clicks into
        // the temporary tmp_tracker_impression_ad_click_connection table
        $query = "
            INSERT INTO
                tmp_tracker_impression_ad_click_connection
                (
                    server_raw_tracker_impression_id,
                    server_raw_ip,
                    tracker_id,
                    campaign_id,
                    operation_interval,
                    operation_interval_id,
                    interval_start,
                    interval_end,
                    viewer_id,
                    date_time,
                    connection_type,
                    connection_window,
                    conversion
                )
            SELECT
                drti.server_raw_tracker_impression_id AS server_raw_tracker_impression_id,
                drti.server_raw_ip AS server_raw_ip,
                drti.tracker_id AS tracker_id,
                ct.campaignid AS campaign_id,
                {$this->conf['maintenance']['operation_interval']} AS operation_interval,
                $operationIntervalID AS operation_interval_id,
                '".$operationIntervalStart->format('%Y-%m-%d %H:%M:%S')."' AS interval_start,
                '".$operationIntervalEnd->format('%Y-%m-%d %H:%M:%S')."' AS interval_end,
                drti.viewer_id AS viewer_id,
                drti.date_time AS date_time,
                '1' AS connection_type,
                ct.clickwindow AS connection_window,
                ct.logstats AS conversion
            FROM
                {$this->conf['table']['data_raw_tracker_impression']} AS drti,        
                {$this->conf['table']['campaigns_trackers']} AS ct
            WHERE        
                drti.tracker_id = ct.trackerid
                AND drti.viewer_id != ''
                AND drti.date_time >= ".$start->format('%Y%m%d%H%M%S')."
                AND drti.date_time <= ".$end->format('%Y%m%d%H%M%S')."
                AND ct.clickwindow > 0";
        debug('Selecting tracker impressions that may connect to ad clicks into the ' .
              'tmp_tracker_impression_ad_click_connection temporary table', PEAR_LOG_DEBUG);
        $this->execute($query, $result);
        // Connect the tracker impressions with ad clicks, where possible
        $query = "
            INSERT INTO
                tmp_ad_connection
                (
                    server_raw_tracker_impression_id,
                    server_raw_ip,
                    date_time,
                    operation_interval,
                    operation_interval_id,
                    interval_start,
                    interval_end,
                    connection_viewer_id,
                    connection_viewer_session_id,
                    connection_date_time,
                    connection_ad_id,
                    connection_creative_id,
                    connection_zone_id,
                    connection_channel,
                    connection_language,
                    connection_ip_address,
                    connection_host_name,
                    connection_country,
                    connection_https,
                    connection_domain,
                    connection_page,
                    connection_query,
                    connection_referer,
                    connection_search_term,
                    connection_user_agent,
                    connection_os,
                    connection_browser,
                    connection_type,
                    connection_window,
                    conversion
                )
            SELECT
                ttiacc.server_raw_tracker_impression_id AS server_raw_tracker_impression_id,
                ttiacc.server_raw_ip AS server_raw_ip,
                ttiacc.date_time AS date_time,
                ttiacc.operation_interval AS operation_interval,
                ttiacc.operation_interval_id AS operation_interval_id,
                ttiacc.interval_start AS interval_start,
                ttiacc.interval_end AS interval_end,
                drac.viewer_id AS connection_viewer_id,
                drac.viewer_session_id AS connection_viewer_session_id,
                drac.date_time AS connection_date_time,
                drac.ad_id AS connection_ad_id,
                drac.creative_id AS connection_creative_id,
                drac.zone_id AS connection_zone_id,
                drac.channel AS connection_channel,
                drac.language AS connection_language,
                drac.ip_address AS connection_ip_address,
                drac.host_name AS connection_host_name,
                drac.country AS connection_country,
                drac.https AS connection_https,
                drac.domain AS connection_domain,
                drac.page AS connection_page,
                drac.query AS connection_query,
                drac.referer AS connection_referer,
                drac.search_term AS connection_search_term,
                drac.user_agent AS connection_user_agent,
                drac.os AS connection_os,
                drac.browser AS connection_browser,
                ttiacc.connection_type AS connection_type,
                ttiacc.connection_window AS connection_window,
                ttiacc.conversion AS conversion
            FROM
                tmp_tracker_impression_ad_click_connection AS ttiacc,
                {$this->conf['table']['data_raw_ad_click']} AS drac,
                {$this->conf['table']['banners']} AS b,
                {$this->conf['table']['campaigns_trackers']} AS ct
            WHERE
                ttiacc.viewer_id = drac.viewer_id
                AND drac.ad_id = b.bannerid
                AND b.campaignid = ct.campaignid
                AND ct.trackerid = ttiacc.tracker_id
                AND ttiacc.campaign_id = ct.campaignid
                AND ttiacc.date_time < DATE_ADD(drac.date_time, INTERVAL ttiacc.connection_window SECOND)
                AND ttiacc.date_time >= drac.date_time";
        debug('Connecting tracker impressions with ad clicks, where possible', PEAR_LOG_DEBUG);
        $this->execute($query, $result);
        $rows += $this->affectedRows();
        // Drop the temporary table for storing tracker impressions that may connect to ad clicks
        $this->tempTables->dropTempTable('tmp_tracker_impression_ad_click_connection');
        return $rows;
    }

    /**
     * A method to update the intermediate tables with summarised data.
     *
     * @param Date $start The start date/time to save from.
     * @param Date $end The end date/time to save to.
     */
    function saveIntermediate($start, $end)
    {
        // Check the start and end dates
        if (!Maintenance::checkIntervalDates($start, $end, $this->conf['maintenance']['operation_interval'])) {
            return 0;
        }
        // Get the start and end dates of the operation interval ID
        list($operationIntervalStart, $operationIntervalEnd) =
            Maintenance::convertDateToOperationIntervalStartAndEndDates($start);
        // Get the operation interval ID
        $operationIntervalID = Maintenance::convertDateToOperationIntervalID($operationIntervalStart);
        // Save connections to the intermediate tables
        if ($this->conf['modules']['Tracker']) {
            // Select the tracker impression/ad impression and tracker impression/ad click connections
            // that the most recent connection, in the case of duplicate connections per tracker
            $query = "
                CREATE TEMPORARY TABLE
                   tmp_connection_latest
                SELECT
                    tac.server_raw_tracker_impression_id AS server_raw_tracker_impression_id,
                    tac.server_raw_ip AS server_raw_ip,
                    MAX(tac.connection_date_time) AS connection_date_time
                FROM
                    tmp_ad_connection AS tac
                WHERE
                    tac.date_time >= '" . $start->format('%Y-%m-%d %H:%M:%S') . "'
                    AND tac.date_time <= '" . $end->format('%Y-%m-%d %H:%M:%S') . "'
                GROUP BY
                    tac.server_raw_tracker_impression_id, tac.server_raw_ip";
            debug('Selecting the tracker impression/ad impression and/or tracker impression/ad click ' .
                  'connections that have the most recent connection (ie. ad impression and/or click) '.
                  'date_time field', PEAR_LOG_DEBUG);
            $this->execute($query, $result);
            // Mark these "latest" connections
            $query = "
                UPDATE
                    tmp_ad_connection AS tac,
                    tmp_connection_latest AS tcl
                SET
                    tac.latest = 1
                WHERE
                    tac.server_raw_tracker_impression_id = tcl.server_raw_tracker_impression_id
                    AND tac.server_raw_ip = tcl.server_raw_ip
                    AND tac.connection_date_time = tcl.connection_date_time";
            debug('Setting the \'latest connection\' flag in the temporary tmp_connection table',
                PEAR_LOG_DEBUG);
            $this->execute($query, $result);
            $connectionRows = $this->affectedRows();
            // Drop the tmp_connection_latest table
            $this->tempTables->dropTempTable('tmp_connection_latest');
            if ($connectionRows > 0) {
                // Save the "latest" connections and the associated variable values
                $this->_saveConnectionsAndVariableValues($start, $end);
            }
            // Drop the tmp_ad_connection table
            $this->tempTables->dropTempTable('tmp_ad_connection');
        }
        // Summarise requests, impressions, clicks, and conversions (if needed)
        // into the data_intermediate_ad table
        if ($this->conf['modules']['Tracker']) {
            // Join any conversions with the sum of their variable values and
            // save in the tmp_conversions temporary table, and also save those
            // conversions where there are no variable values associated
            $query = "
                CREATE TEMPORARY TABLE
                    tmp_conversions
                SELECT
                    DATE_FORMAT(diac.tracker_date_time, '%Y-%m-%d') AS day,
                    DATE_FORMAT(diac.tracker_date_time, '%k') AS hour,
                    {$this->conf['maintenance']['operation_interval']} AS operation_interval,
                    $operationIntervalID AS operation_interval_id,
                    '".$operationIntervalStart->format('%Y-%m-%d %H:%M:%S')."' AS interval_start,
                    '".$operationIntervalEnd->format('%Y-%m-%d %H:%M:%S')."' AS interval_end,
                    diac.ad_id AS ad_id,        
                    diac.creative_id AS creative_id,   
                    diac.zone_id AS zone_id,
                    SUM(diavv.value) AS basket_value
                FROM
                    {$this->conf['table']['data_intermediate_ad_connection']} AS diac
                LEFT JOIN
                    {$this->conf['table']['data_intermediate_ad_variable_value']} AS diavv
                USING
                    (
                        data_intermediate_ad_connection_id
                    )
                LEFT JOIN
                    {$this->conf['table']['variables']} AS v
                ON
                    (
                        diavv.tracker_variable_id = v.variableid
                    )
                WHERE
                    diac.conversion = 'y'
                    AND v.is_basket_value = 1
                    AND diac.tracker_date_time >= '" . $start->format('%Y-%m-%d %H:%M:%S') . "'
                    AND diac.tracker_date_time <= '" . $end->format('%Y-%m-%d %H:%M:%S') . "'
                GROUP BY
                    day, hour, ad_id, creative_id, zone_id
                UNION ALL
                SELECT
                    DATE_FORMAT(diac.tracker_date_time, '%Y-%m-%d') AS day,
                    DATE_FORMAT(diac.tracker_date_time, '%k') AS hour,
                    {$this->conf['maintenance']['operation_interval']} AS operation_interval,
                    $operationIntervalID AS operation_interval_id,
                    '".$operationIntervalStart->format('%Y-%m-%d %H:%M:%S')."' AS interval_start,
                    '".$operationIntervalEnd->format('%Y-%m-%d %H:%M:%S')."' AS interval_end,
                    diac.ad_id AS ad_id,        
                    diac.creative_id AS creative_id,   
                    diac.zone_id AS zone_id,
                    0 AS basket_value
                FROM
                    {$this->conf['table']['data_intermediate_ad_connection']} AS diac
                LEFT JOIN
                    {$this->conf['table']['data_intermediate_ad_variable_value']} AS diavv
                USING
                    (
                        data_intermediate_ad_connection_id
                    )
                WHERE
                    diac.conversion = 'y'
                    AND diavv.data_intermediate_ad_connection_id IS NULL
                    AND diac.tracker_date_time >= '" . $start->format('%Y-%m-%d %H:%M:%S') . "'
                    AND diac.tracker_date_time <= '" . $end->format('%Y-%m-%d %H:%M:%S') . "'";
            debug('Joining the conversions with their basket values', PEAR_LOG_DEBUG);
            $this->execute($query, $result);
            // Create a union table of the requests, impressions, clicks and conversions
            $query = "
                CREATE TEMPORARY TABLE
                    tmp_union
                SELECT
                    tar.day AS day,
                    tar.hour AS hour,
                    tar.operation_interval AS operation_interval,
                    tar.operation_interval_id AS operation_interval_id,
                    tar.interval_start AS interval_start,
                    tar.interval_end AS interval_end,
                    tar.ad_id AS ad_id,
                    tar.creative_id AS creative_id,
                    tar.zone_id AS zone_id,
                    tar.requests AS requests,
                    0 AS impressions,
                    0 AS clicks,
                    0 AS conversions,
                    0 AS total_basket_value
                FROM
                    tmp_ad_request AS tar
                UNION ALL
                SELECT
                    tai.day AS day,
                    tai.hour AS hour,
                    tai.operation_interval AS operation_interval,
                    tai.operation_interval_id AS operation_interval_id,
                    tai.interval_start AS interval_start,
                    tai.interval_end AS interval_end,
                    tai.ad_id AS ad_id,
                    tai.creative_id AS creative_id,
                    tai.zone_id AS zone_id,
                    0 AS requests,
                    tai.impressions AS impressions,
                    0 AS clicks,
                    0 AS conversions,
                    0 AS total_basket_value
                FROM
                    tmp_ad_impression AS tai
                UNION ALL
                SELECT
                    tac.day AS day,
                    tac.hour AS hour,
                    tac.operation_interval AS operation_interval,
                    tac.operation_interval_id AS operation_interval_id,
                    tac.interval_start AS interval_start,
                    tac.interval_end AS interval_end,
                    tac.ad_id AS ad_id,
                    tac.creative_id AS creative_id,
                    tac.zone_id AS zone_id,
                    0 AS requests,
                    0 AS impressions,
                    tac.clicks AS clicks,
                    0 AS conversions,
                    0 AS total_basket_value
                FROM
                    tmp_ad_click AS tac 
                UNION ALL
                SELECT
                    tc.day AS day,
                    tc.hour AS hour,
                    tc.operation_interval AS operation_interval,
                    tc.operation_interval_id AS operation_interval_id,
                    tc.interval_start AS interval_start,
                    tc.interval_end AS interval_end,
                    tc.ad_id AS ad_id,
                    tc.creative_id AS creative_id,
                    tc.zone_id AS zone_id,
                    0 AS requests,
                    0 AS impressions,
                    0 AS clicks,
                    COUNT(*) AS conversions,
                    SUM(tc.basket_value) AS total_basket_value
                FROM
                    tmp_conversions AS tc
                GROUP BY
                    day, hour, ad_id, creative_id, zone_id";
            debug('Creating a union of the requests, impressions, clicks and conversions', PEAR_LOG_DEBUG);
            $this->execute($query, $result);
            // Drop the tmp_conversions table
            $this->tempTables->dropTempTable('tmp_conversions');
            $table = $this->conf['table']['data_intermediate_ad'];
            $query = "
                INSERT INTO
                    $table
                    (
                        day,
                        hour,
                        operation_interval,
                        operation_interval_id,
                        interval_start,
                        interval_end,
                        ad_id,
                        creative_id,
                        zone_id,
                        requests,
                        impressions,
                        clicks,
                        conversions,
                        total_basket_value
                    )
                SELECT
                    tu.day AS day,
                    tu.hour AS hour,
                    tu.operation_interval AS operation_interval,
                    tu.operation_interval_id AS operation_interval_id,
                    tu.interval_start AS interval_start,
                    tu.interval_end AS interval_end,
                    tu.ad_id AS ad_id,
                    tu.creative_id AS creative_id,
                    tu.zone_id AS zone_id,
                    SUM(tu.requests) AS requests,
                    SUM(tu.impressions) AS impressions,
                    SUM(tu.clicks) AS clicks,
                    SUM(tu.conversions) AS conversions,
                    SUM(tu.total_basket_value) AS total_basket_value
                FROM
                    tmp_union AS tu
                GROUP BY
                    day, hour, ad_id, creative_id, zone_id";
            debug("Inserting the ad requests, impressions, clicks, and conversions into the $table table",
                PEAR_LOG_DEBUG);
            $this->execute($query, $result);
            // Drop the tmp_union table
            $this->tempTables->dropTempTable('tmp_union');
        } else {
            // Tracker module not installed, just summarise requests, impressions
            // and clicks: Create a union table of the requests, impressions and clicks
            $query = "
                CREATE TEMPORARY TABLE
                    tmp_union
                SELECT
                    tar.day AS day,
                    tar.hour AS hour,
                    tar.operation_interval AS operation_interval,
                    tar.operation_interval_id AS operation_interval_id,
                    tar.interval_start AS interval_start,
                    tar.interval_end AS interval_end,
                    tar.ad_id AS ad_id,
                    tar.creative_id AS creative_id,
                    tar.zone_id AS zone_id,
                    tar.requests AS requests,
                    0 AS impressions,
                    0 AS clicks
                FROM
                    tmp_ad_request AS tar
                UNION ALL
                SELECT
                    tai.day AS day,
                    tai.hour AS hour,
                    tai.operation_interval AS operation_interval,
                    tai.operation_interval_id AS operation_interval_id,
                    tai.interval_start AS interval_start,
                    tai.interval_end AS interval_end,
                    tai.ad_id AS ad_id,
                    tai.creative_id AS creative_id,
                    tai.zone_id AS zone_id,
                    0 AS requests,
                    tai.impressions AS impressions,
                    0 AS clicks
                FROM
                    tmp_ad_impression AS tai
                UNION ALL
                SELECT
                    tac.day AS day,
                    tac.hour AS hour,
                    tac.operation_interval AS operation_interval,
                    tac.operation_interval_id AS operation_interval_id,
                    tac.interval_start AS interval_start,
                    tac.interval_end AS interval_end,
                    tac.ad_id AS ad_id,
                    tac.creative_id AS creative_id,
                    tac.zone_id AS zone_id,
                    0 AS requests,
                    0 AS impressions,
                    tac.clicks AS clicks
                FROM
                    tmp_ad_click AS tac 
                GROUP BY
                    day, hour, ad_id, creative_id, zone_id";
            debug('Creating a union of the requests, impressions and clicks', PEAR_LOG_DEBUG);
            $this->execute($query, $result);
            $table = $this->conf['table']['data_intermediate_ad'];
            $query = "
                INSERT INTO
                    $table
                    (
                        day,
                        hour,
                        operation_interval,
                        operation_interval_id,
                        interval_start,
                        interval_end,
                        ad_id,
                        creative_id,
                        zone_id,
                        requests,
                        impressions,
                        clicks,
                        conversions,
                        total_basket_value
                    )
                SELECT
                    tu.day AS day,
                    tu.hour AS hour,
                    tu.operation_interval AS operation_interval,
                    tu.operation_interval_id AS operation_interval_id,
                    tu.interval_start AS interval_start,
                    tu.interval_end AS interval_end,
                    tu.ad_id AS ad_id,
                    tu.creative_id AS creative_id,
                    tu.zone_id AS zone_id,
                    SUM(tu.requests) AS requests,
                    SUM(tu.impressions) AS impressions,
                    SUM(tu.clicks) AS clicks,
                    '0' AS conversions,
                    '0' AS total_basket_value
                FROM
                    tmp_union AS tu
                GROUP BY
                    day, hour, ad_id, creative_id, zone_id";
            debug("Inserting the ad requests, impressions and clicks into the $table table", PEAR_LOG_DEBUG);
            $this->execute($query, $result);
            // Drop the tmp_union table
            $this->tempTables->dropTempTable('tmp_union');
        }
        // Drop the tmp_ad_request table
        $this->tempTables->dropTempTable('tmp_ad_request');
        // Drop the tmp_ad_impression table
        $this->tempTables->dropTempTable('tmp_ad_impression');
        // Drop the tmp_ad_click table
        $this->tempTables->dropTempTable('tmp_ad_click');
    }
    
    /**
     * A private method to save the tacker impression/ad impression and
     * tracker impression/ad click connections that have been marked as
     * "latest", and also save the associated tracker impression 
     * variable values.
     *
     * @private
     * @param Date $start The start date/time to save from.
     * @param Date $end The end date/time to save to.
     */
    function _saveConnectionsAndVariableValues($start, $end)
    {
        // Save the connections
        $table = $this->conf['table']['data_intermediate_ad_connection'];
        $query = "
            INSERT INTO
                $table
                (
                    server_raw_tracker_impression_id,
                    server_raw_ip,
                    viewer_id,
                    viewer_session_id,
                    tracker_date_time,
                    connection_date_time,
                    tracker_id,
                    ad_id,
                    creative_id,
                    zone_id,
                    tracker_channel,
                    connection_channel,
                    tracker_language,
                    connection_language,
                    tracker_ip_address,
                    connection_ip_address,
                    tracker_host_name,
                    connection_host_name,
                    tracker_country,
                    connection_country,
                    tracker_https,
                    connection_https,
                    tracker_domain,
                    connection_domain,
                    tracker_page,
                    connection_page,
                    tracker_query,
                    connection_query,
                    tracker_referer,
                    connection_referer,
                    tracker_search_term,
                    connection_search_term,
                    tracker_user_agent,
                    connection_user_agent,
                    tracker_os,
                    connection_os,
                    tracker_browser,
                    connection_browser,
                    connection_type,
                    connection_window,
                    conversion
                )
            SELECT
                drti.server_raw_tracker_impression_id AS server_raw_tracker_impression_id,
                drti.server_raw_ip AS server_raw_ip,
                drti.viewer_id AS viewer_id,
                drti.viewer_session_id AS viewer_session_id,
                drti.date_time AS tracker_date_time,
                tac.connection_date_time AS connection_date_time,
                drti.tracker_id AS tracker_id,
                tac.connection_ad_id AS ad_id,
                tac.connection_creative_id AS creative_id,
                tac.connection_zone_id AS zone_id,
                drti.channel AS tracker_channel,
                tac.connection_channel AS connection_channel,
                drti.language AS tracker_language,
                tac.connection_language AS connection_language,
                drti.ip_address AS tracker_ip_address,
                tac.connection_ip_address AS connection_ip_address,
                drti.host_name AS tracker_host_name,
                tac.connection_host_name AS connection_host_name,
                drti.country AS tracker_country,
                tac.connection_country AS connection_country,
                drti.https AS tracker_https,
                tac.connection_https AS connection_https,
                drti.domain AS tracker_domain,
                tac.connection_domain AS connection_domain,
                drti.page AS tracker_page,
                tac.connection_page AS connection_page,
                drti.query AS tracker_query,
                tac.connection_query AS connection_query,
                drti.referer AS tracker_referer,
                tac.connection_referer AS connection_referer,
                drti.search_term AS tracker_search_term,
                tac.connection_search_term AS connection_search_term,
                drti.user_agent AS tracker_user_agent,
                tac.connection_user_agent AS connection_user_agent,
                drti.os AS tracker_os,
                tac.connection_os AS connection_os,
                drti.browser AS tracker_browser,
                tac.connection_browser AS connection_browser,
                tac.connection_type AS connection_type,
                tac.connection_window AS connection_window,
                tac.conversion AS conversion
            FROM
                tmp_ad_connection AS tac,
                {$this->conf['table']['data_raw_tracker_impression']} AS drti
            WHERE
                tac.server_raw_tracker_impression_id = drti.server_raw_tracker_impression_id
                AND tac.server_raw_ip = drti.server_raw_ip
                AND tac.latest = 1";
        debug("Inserting the connections into the $table table", PEAR_LOG_DEBUG);
        $this->execute($query, $result);
        // Save the variable values
        $table = $this->conf['table']['data_intermediate_ad_variable_value'];
        $query = "
            INSERT INTO
                $table
                (
                    data_intermediate_ad_connection_id,
                    tracker_variable_id,
                    value
                )
            SELECT
                diac.data_intermediate_ad_connection_id AS data_intermediate_ad_connection_id,        
                drtvv.tracker_variable_id AS tracker_variable_id,        
                drtvv.value AS value
            FROM
                {$this->conf['table']['data_intermediate_ad_connection']} AS diac,
                {$this->conf['table']['data_raw_tracker_variable_value']} AS drtvv,
                {$this->conf['table']['data_raw_tracker_impression']} AS drti
            WHERE
                diac.server_raw_tracker_impression_id = drti.server_raw_tracker_impression_id
                AND diac.server_raw_ip = drti.server_raw_ip
                AND drti.server_raw_tracker_impression_id = drtvv.server_raw_tracker_impression_id
                AND drti.server_raw_ip = drtvv.server_raw_ip
                AND diac.tracker_date_time >= '" . $start->format('%Y-%m-%d %H:%M:%S') . "'
                AND diac.tracker_date_time <= '" . $end->format('%Y-%m-%d %H:%M:%S') . "'";
        debug("Saving the tracker impression variable values from the $table table", PEAR_LOG_DEBUG);
        $this->execute($query, $result);
    }
    
    /**
     * A method to update the zone impression history table from the intermediate tables.
     *
     * @param Date $start The start date/time to update from.
     * @param Date $end The end date/time to update to.
     */
    function saveHistory($start, $end)
    {
        $fromTable = $this->conf['table']['data_intermediate_ad'];
        $toTable   = $this->conf['table']['data_summary_zone_impression_history'];
        $query = "
            SELECT
                operation_interval AS operation_interval,
                operation_interval_id AS operation_interval_id,
                interval_start AS interval_start,
                interval_end AS interval_end,
                zone_id AS zone_id,
                SUM(impressions) AS actual_impressions
            FROM
                $fromTable
            WHERE
                interval_start >= '".$start->format('%Y-%m-%d %H:%M:%S')."'
                AND interval_end <= '".$end->format('%Y-%m-%d %H:%M:%S')."'
            GROUP BY
                zone_id";
        debug('Selecting total zone impressions from the ' . $fromTable . ' table for data >= ' .
              $start->format('%Y-%m-%d %H:%M:%S') . ', and <= ' . $end->format('%Y-%m-%d %H:%M:%S'),
              PEAR_LOG_DEBUG);
        $this->execute($query, $result);
        while ($row = $this->fetchResultAsArray($result)) {
            $query = "
                UPDATE
                    $toTable
                SET
                    actual_impressions = {$row['actual_impressions']}
                WHERE
                    operation_interval = {$row['operation_interval']}
                    AND operation_interval_id = {$row['operation_interval_id']}
                    AND interval_start = '{$row['interval_start']}'
                    AND interval_end = '{$row['interval_end']}'
                    AND zone_id = {$row['zone_id']}";
            $this->execute($query, $innerResult);
            if ($this->affectedRows() == 0) {
                // Unable to UPDATE, try INSERT instead
                $query = "
                    INSERT INTO
                        $toTable
                        (
                            operation_interval,
                            operation_interval_id,
                            interval_start,
                            interval_end,
                            zone_id,
                            actual_impressions
                        )
                    VALUES
                        (
                            {$row['operation_interval']},
                            {$row['operation_interval_id']},
                            '{$row['interval_start']}',
                            '{$row['interval_end']}',
                            {$row['zone_id']},
                            {$row['actual_impressions']}
                        )";
                $this->execute($query, $innerResult);
            }
        }
    }
    
    /**
     * A method to update the summary table from the intermediate tables.
     *
     * @param Date $start The start date/time to update from.
     * @param Date $end The end date/time to update to.
     */
    function saveSummary($start, $end)
    {
        $fromTable = $this->conf['table']['data_intermediate_ad'];
        $toTable   = $this->conf['table']['data_summary_ad_hourly'];
        $query = "
            INSERT INTO
                $toTable
                (
                    day,
                    hour,
                    ad_id,
                    creative_id,
                    zone_id,
                    requests,
                    impressions,
                    clicks,
                    conversions,
                    total_basket_value
                )
            SELECT
                day AS day,
                hour AS hour,
                ad_id AS ad_id,
                creative_id AS creative_id,
                zone_id AS zone_id,
                SUM(requests) AS requests,
                SUM(impressions) AS impressions,
                SUM(clicks) AS clicks,
                SUM(conversions) AS conversions,
                SUM(total_basket_value) AS total_basket_value
            FROM
                $fromTable
            WHERE
                day >= '".$start->format('%Y-%m-%d')."'
                AND hour >= ".$start->format('%H')."
                AND day <= '".$end->format('%Y-%m-%d')."'
                AND hour <= ".$end->format('%H')."
        
            GROUP BY
                day, hour, ad_id, creative_id, zone_id";
        debug('Summarising ad requests, impressions, clicks and conversions into the ' . $toTable . 
              ' table from the ' . $fromTable . ' table for data >= day ' . $start->format('%Y-%m-%d') .
              ', hour ' . $start->format('%H') . ', and <= day ' . $end->format('%Y-%m-%d') . ', hour ' .
              $end->format('%H'), PEAR_LOG_DEBUG);
        $this->execute($query, $result);
    }

    /**
     * A method to decrement the remaining impression, click and conversion
     * targets of campaigns based on the saved, summarised data.
     *
     * @param string $report A reference to the report to be logged.
     * @param Date $sdecrStart The date from which impressions, clicks and conversions
     *                         should be used to decrement the campaign targets.
     * @param Date $decrEnd The date up to which impressions, clicks and conversions
     *                      should be used to decrement the campaign targets.
     */
    function decrementCampaigns(&$report, $decrStart, $decrEnd)
    {
        $report .= "\n";
        $query = "
            SELECT
                ca.campaignid AS campaign_id,
                ca.campaignname AS campaign_name,
                cl.contact AS contact,
                cl.email AS email,
                ca.views AS impressions,
                ca.clicks AS clicks,
                ca.conversions AS conversions,
                ca.active AS active,
                ca.activate AS start,
                ca.expire AS end,
                NOW() AS now
            FROM
                {$this->conf['table']['campaigns']} AS ca,
                {$this->conf['table']['clients']} AS cl
            WHERE
                ca.clientid = cl.clientid";
        debug("Selecting all campaigns", PEAR_LOG_DEBUG);
        $this->execute($query, $result);
        $dbNow = new Date();        
        while ($campaignRow = $this->fetchResultAsArray($result)) {
            if ($campaignRow['active'] == 't') {
                // The campaign is currently active, look at the campaign
                $disableReason = 0;
                if (($campaignRow['impressions'] > 0) ||
                    ($campaignRow['clicks'] > 0) ||
                    ($campaignRow['conversions'] > 0)) {
                    // The campaign has an impression, click and/or conversion target, 
                    // so look to see if there are any new statistics for the campaign
                    $query = "
                        SELECT
                            SUM(dia.impressions) AS impressions,
                            SUM(dia.clicks) AS clicks,
                            SUM(dia.conversions) AS conversions
                        FROM
                            {$this->conf['table']['data_intermediate_ad']} AS dia,
                            {$this->conf['table']['banners']} AS b
                        WHERE
                            dia.ad_id = b.bannerid
                            AND b.campaignid = {$campaignRow['campaign_id']}
                            AND dia.interval_start >= '".$decrStart->format('%Y-%m-%d %H:%M:%S')."'
                            AND dia.interval_end <= '".$decrEnd->format('%Y-%m-%d %H:%M:%S')."'";
                    $this->execute($query, $innerResult);
                    $valuesRow = $this->fetchResultAsArray($innerResult);
                    if ((!is_null($valuesRow['impressions'])) || (!is_null($valuesRow['clicks'])) || (!is_null($valuesRow['conversions']))) {
                        // There were impressions, clicks and/or conversions for this
                        // campaign, so find out if campaign targets have been passed
                        if (is_null($valuesRow['impressions'])) {
                            // No impressions
                            $valuesRow['impressions'] = 0;
                        }
                        if (is_null($valuesRow['clicks'])) {
                            // No clicks
                            $valuesRow['clicks'] = 0;
                        }
                        if (is_null($valuesRow['conversions'])) {
                            // No conversions
                            $valuesRow['conversions'] = 0;
                        }
                        if ($campaignRow['impressions'] > 0) {
                            if ($campaignRow['impressions'] <= $valuesRow['impressions']) {
                                // The campaign has an impressions target, and this has been
                                // passed, so update and disable the campagin
                                $disableReason = 4;
                                // Ensure that the final value in the campaigns table is zero
                                $valuesRow['impressions'] = $campaignRow['impressions'];
                            }
                        } else {
                            // Ensure the value remains -1
                            $valuesRow['impressions'] = 0;
                        }
                        if ($campaignRow['clicks'] > 0) {
                            if ($campaignRow['clicks'] <= $valuesRow['clicks']) {
                                // The campaign has a click target, and this has been
                                // passed, so update and disable the campagin
                                $disableReason = 4;
                                // Ensure that the final value in the campaigns table is zero
                                $valuesRow['clicks'] = $campaignRow['clicks'];
                            }
                        } else {
                            // Ensure the value remains -1
                            $valuesRow['clicks'] = 0;
                        }
                        if ($campaignRow['conversions'] > 0) {
                            if ($campaignRow['conversions'] <= $valuesRow['conversions']) {
                                // The campaign has a target limitation, and this has been
                                // passed, so update and disable the campagin
                                $disableReason |= 8;
                                // Ensure that the final value in the campaigns table is zero
                                $valuesRow['conversions'] = $campaignRow['conversions'];
                            }
                        } else {
                            // Ensure the value remains -1
                            $valuesRow['conversions'] = 0;                            
                        }
                        if ($disableReason) {
                            // One of the campaign targets was exceeded, so disable
                            $query = "
                                UPDATE
                                    {$this->conf['table']['campaigns']}
                                SET
                                    views = {$campaignRow['impressions']} - {$valuesRow['impressions']},
                                    clicks = {$campaignRow['clicks']} - {$valuesRow['clicks']},
                                    conversions = {$campaignRow['conversions']} - {$valuesRow['conversions']},
                                    active = 'f'
                                WHERE
                                    campaignid = {$campaignRow['campaign_id']}";
                            $report .= ' - Exceeded a campaign quota: Deactivating campaign ID ' .
                                  "{$campaignRow['campaign_id']}: {$campaignRow['campaign_name']}\n";
                            debug('Exceeded a campaign quota: Deactivating campaign ID ' .
                                  "{$campaignRow['campaign_id']}: {$campaignRow['campaign_name']}");
                            $this->execute($query, $innerResult);
                            phpAds_userlogAdd(phpAds_actionDeactiveCampaign, $campaignRow['campaign_id']);
                        } else {
                            // No campaign limits were exceeded, so just update the
                            // current limitation requirements
                            $query = "
                                UPDATE
                                    {$this->conf['table']['campaigns']}
                                SET
                                    views = views - {$valuesRow['impressions']},
                                    clicks = clicks - {$valuesRow['clicks']},
                                    conversions = conversions - {$valuesRow['conversions']}
                                WHERE
                                    campaignid = {$campaignRow['campaign_id']}";
                            debug("Updating campaign quotas: Campaign ID " .
                                  "{$campaignRow['campaign_id']}: {$campaignRow['campaign_name']}", PEAR_LOG_DEBUG);
                            $this->execute($query, $innerResult);
                            
                        }
                    }
                }
                // Does the campaign need to be disabled due to the date?
                //$dbNow = new Date($campaignRow['now']);
                $end = new Date($campaignRow['end'] . ' 23:59:59');  // Convert day to end of day Date
                if (($end->format('%Y-%m-%d %H:%M:%S') != '0000-00-00 00:00:00') && ($dbNow->after($end))) {
                    $disableReason |= 16;
                    $query = "
                        UPDATE
                            {$this->conf['table']['campaigns']}
                        SET
                            active = 'f'
                        WHERE
                            campaignid = {$campaignRow['campaign_id']}";
                    $report .= ' - Past campaign end time: Deactivating campaign ID ' .
                          "{$campaignRow['campaign_id']}: {$campaignRow['campaign_name']}\n";
                    debug('Found campaign end date of ' . $end->format('%Y-%m-%d %H:%M:%S') . ' has been '.
                          'passed by current time of ' . $dbNow->format('%Y-%m-%d %H:%M:%S'), PEAR_LOG_DEBUG);
                    debug('Past campaign end time: Deactivating campaign ID ' .
                          "{$campaignRow['campaign_id']}: {$campaignRow['campaign_name']}", PEAR_LOG_INFO);
                    $this->execute($query, $innerResult);
                    phpAds_userlogAdd(phpAds_actionDeactiveCampaign, $campaignRow['campaign_id']);
                }
                if ($disableReason) {
                    // The campaign was disabled, so send the appropriate
                    // message to the campaign's contact
                    $query = "
                        SELECT
                            bannerid AS advertisement_id,
                            description AS description,
                            alt AS alt,
                            url AS url
                        FROM
                            {$this->conf['table']['banners']}
                        WHERE
                            campaignid = {$campaignRow['campaign_id']}";
                    debug("Getting the advertisements for campaign ID {$campaignRow['campaign_id']}", PEAR_LOG_DEBUG);
                    $this->execute($query, $advertisementResult);
                    while ($advertisementRow = $this->fetchResultAsArray($advertisementResult)) {
                        $advertisements[$advertisementRow['advertisement_id']] = 
                            array($advertisementRow['description'], $advertisementRow['alt'],
                                  $advertisementRow['url']);
                    }
                    if ($this->conf['email']['sendMail']) {
                        $message =& Maintenance::prepareDeactivateCampaignEmail($campaignRow['contact'],
                                                                                $campaignRow['campaign_name'],
                                                                                $disableReason, $advertisements);
                        phpAds_sendMail($campaignRow['email'], $campaignRow['contact'],
                                        "Deactivated Banners: {$campaignRow['campaign_name']}", $message);
                    }
                }
            } else {                
                // The campaign is not active - does it need to be enabled,
                // based on the campaign starting date?
                //$dbNow = new Date($campaignRow['now']);
                $start = new Date($campaignRow['start'] . ' 00:00:00');  // Convert day to start of date Date
                $end   = new Date($campaignRow['end']   . ' 23:59:59');  // Convert day to end of day Date
                if (($start->format('%Y-%m-%d %H:%M:%S') != '0000-00-00 00:00:00') && ($dbNow->after($start))) {
                    // There is an activation date, which has been passed. To activate,
                    // check that none of the campaign limitation requirements have been
                    // met, and if not, ensure that either there is no end date, or that
                    // the end date has not passed
                    if (($campaignRow['impressions'] != 0) && ($campaignRow['clicks'] != 0) &&
                        ($campaignRow['conversions'] != 0) &&
                        (($end->format('%Y-%m-%d %H:%M:%S') == '0000-00-00 00:00:00') ||
                        (($end->format('%Y-%m-%d %H:%M:%S') != '0000-00-00 00:00:00') &&
                        (Date::compare($dbNow, $end) < 0)))) {
                        $query = "
                            UPDATE
                                {$this->conf['table']['campaigns']}
                            SET
                                active = 't'
                            WHERE
                                campaignid = {$campaignRow['campaign_id']}";
                        $report .= ' - Past campaign start time: Activating campaign ID ' .
                              "{$campaignRow['campaign_id']}: {$campaignRow['campaign_name']}\n";
                        debug('Found campaign start date of ' . $start->format('%Y-%m-%d %H:%M:%S') . ' has been '.
                              'passed by current time of ' . $dbNow->format('%Y-%m-%d %H:%M:%S'), PEAR_LOG_DEBUG);
                        debug('Past campaign start time: Activating campaign ID ' .
                              "{$campaignRow['campaign_id']}: {$campaignRow['campaign_name']}", PEAR_LOG_INFO);
                        $this->execute($query, $innerResult);
                        phpAds_userlogAdd(phpAds_actionActiveCampaign, $campaignRow['campaign_id']);
                        // Get the advertisements associated with the campaign
                        $query = "
                            SELECT
                                bannerid AS advertisement_id,
                                description AS description,
                                alt AS alt,
                                url AS url
                            FROM
                                {$this->conf['table']['banners']}
                            WHERE
                                campaignid = {$campaignRow['campaign_id']}";
                        debug("Getting the advertisements for campaign ID {$campaignRow['campaign_id']}",
                              PEAR_LOG_DEBUG);
                        $this->execute($query, $advertisementResult);
                        while ($advertisementRow = $this->fetchResultAsArray($advertisementResult)) {
                            $advertisements[$advertisementRow['advertisement_id']] =
                                array($advertisementRow['description'], $advertisementRow['alt'],
                                    $advertisementRow['url']);
                        }
                        if ($this->conf['email']['sendMail']) {
                            $message =& Maintenance::prepareActivateCampaignEmail($campaignRow['contact'],
                                                                                  $campaignRow['campaign_name'],
                                                                                  $advertisements);
                            phpAds_sendMail($campaignRow['email'], $campaignRow['contact'],
                                            "Activated Banners: {$campaignRow['campaign_name']}", $message);
                        }
                    }
                }
            }
        }
    }
    
    /**
     * A method to delete old (ie. summarised) raw data.
     *
     * @param Date $summarisedTo The date/time up to which data have been summarised (i.e. data up
     *                           to and including this date (minus any compact_stats_grace window)
     *                           will be deleted, unless required by the tracking module, where
     *                           installed).
     * @return integer The number of rows deleted.
     */
    function deleteOldData($summarisedTo)
    {
        $deleteDate = $summarisedTo;
        if ($this->conf['maintenance']['compact_stats_grace'] > 0) {
            $deleteDate->subtractSeconds((int) $this->conf['maintenance']['compact_stats_grace']);
        }
        $rows = 0;
        // Delete the ad requests before taking into account the maximum connection window
        $table = $this->conf['table']['data_raw_ad_request'];
        $query = "
            DELETE FROM
                $table
            WHERE
                date_time <= '" . $deleteDate->format('%Y-%m-%d %H:%M:%S') ."'";
        debug("Deleting summarised (earlier than '" . $deleteDate->format('%Y-%m-%d %H:%M:%S') .
              "') ad requests from the $table table", PEAR_LOG_DEBUG);
        $this->execute($query, $result);
        $rows += $this->affectedRows();
        // Take into account the maximum connection window, if approproate
        if ($this->conf['modules']['Tracker']) {
            // Find the largest, active impression and click connection windows
            list($impressionWindow, $clickWindow) = Maintenance::maxConnectionWindows();
            // Find the largest of the two windows
            if ($impressionWindow > $clickWindow) {
                $maxWindow = $impressionWindow;
            } else {
                $maxWindow = $clickWindow;
            }
            debug('Found maximum connection window of ' . $maxWindow . ' seconds', PEAR_LOG_DEBUG);
            $deleteDate->subtractSeconds((int) $maxWindow); // Cast to int, as Date class
                                                            // doesn't deal with strings
        }
        // Delete the ad impressions
        $table = $this->conf['table']['data_raw_ad_impression'];
        $query = "
            DELETE FROM
                $table
            WHERE
                date_time <= '" . $deleteDate->format('%Y-%m-%d %H:%M:%S') ."'";
        debug("Deleting summarised (earlier than '" . $deleteDate->format('%Y-%m-%d %H:%M:%S') .
              "') ad impressions from the $table table", PEAR_LOG_DEBUG);
        $this->execute($query, $result);
        $rows += $this->affectedRows();
        // Delete the ad clicks
        $table = $this->conf['table']['data_raw_ad_click'];
        $query = "
            DELETE FROM
                $table
            WHERE
                date_time <= '" . $deleteDate->format('%Y-%m-%d %H:%M:%S') ."'";
        debug("Deleting summarised (earlier than '" . $deleteDate->format('%Y-%m-%d %H:%M:%S') .
              "') ad clicks from the $table table", PEAR_LOG_DEBUG);
        $this->execute($query, $result);
        $rows += $this->affectedRows();
        return $rows;
    }
    
    /**
     * A method to store the details logging a maintenance statistics
     * run.
     *
     * @param Date $start The time that the maintenance statistics run
     *                    started.
     * @param Date $end The time that the maintenance statistics run
     *                  ended.
     * @param integer $type The type of run:
     *                          0 - The update was based on the operation interval;
     *                          1 - The update was based on the hour;
     *                          2 - The update was based on both the operation interval and hour.
     * @param Date $updateTo The end of the operation interval/hour that has been updated.
     * @param string $report The maintenance statistics run report.
     */
    function logCompletion($start, $end, $type, $updateTo, &$report) {
        $duration = new Date_Span();
        $duration->setFromDateDiff($start, $end);
        $query = "
            INSERT INTO
                {$this->conf['table']['log_maintenance_statistics']}
                (
                    start_run,
                    end_run,
                    duration,
                    adserver_run_type,
                    updated_to
                )
            VALUES
                (
                    '".$start->format('%Y-%m-%d %H:%M:%S')."',
                    '".$end->format('%Y-%m-%d %H:%M:%S')."',
                    ".$duration->toSeconds().",
                    $type,
                    '".$updateTo->format('%Y-%m-%d %H:%M:%S')."'
                )";
        debug('Logging the maintenance statistics run information', PEAR_LOG_DEBUG);
        $this->execute($query, $result);
        $query = "
            INSERT INTO
                {$this->conf['table']['userlog']}
                (
                    timestamp,
                    usertype,
                    userid,
                    action,
                    object,
                    details
                )
            VALUES
                (
                    '".time()."',
                    '".phpAds_userMaintenance."',
                    0,
                    '".phpAds_actionBatchStatistics."',
                    0,
                    '".addslashes($report)."'
                )";
        debug('Logging the maintenance statistics run report', PEAR_LOG_DEBUG);
        $this->execute($query, $result);
    }
    
}

?>
